#include <jni.h>
#include "edu_stanford_db_xsb_XSBCore.h"
#include "hashtable.h"
#include <stdio.h>
#include <cinterf.h>
#include <string.h>
#include <malloc.h>
#include "javaxsbdefs.h"
#include <varstring_xsb.h>

JNIEnv * currentEnv = 0;

HashTable * initVarHashTable(){
  HashTable* hashTable = HashTableCreate(15);
  HashTableSetHashFunction(hashTable, HashTableStringHashFunction);
  HashTableSetKeyComparisonFunction(hashTable, strcmp );
  HashTableSetDeallocationFunctions(hashTable, free, NULL);
  return hashTable;
}



/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_init
 * Signature: ([Ljava/lang/String;)I
 */


JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1init(JNIEnv * env, jobject obj, jobjectArray array){
    /* get length of array */
    jsize len = (*env)->GetArrayLength(env, array);  
    /* declare array */
    char * myargv[len];
    int i;
    jint rcode;
    currentEnv = env;
    for(i=0;i<len;i++){
	/* get ith element (we know it is a jstring) */
	jstring string = (jstring)(*env)->GetObjectArrayElement(env, array, (jsize) i );
	/* get the UTF equivalent */
	myargv[i] = (char *) (*env)->GetStringUTFChars(env, string, 0);
    }
    rcode = xsb_init(len,myargv);  /* depend on user to put in right options (-n) */
    for(i=0;i<len;i++){
	jstring string = (jstring)(*env)->GetObjectArrayElement(env, array, (jsize) i );
	/* release memory again */
	(*env)->ReleaseStringUTFChars(env, string, myargv[i]);
    }
    if(getPredicateSymbolLiteral==0) initReferences(env);
    return rcode; 
}


/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_close
 * Signature: ()I
 */

JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1close(JNIEnv * env, jobject obj){
    jint rcode = (jint) xsb_close();
    return rcode;
}


/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_command_string
 * Signature: (Ljava/lang/String;)I
 */

JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1command_1string
    (JNIEnv * env, jobject obj, jstring string){
    char * command = (char *) (*env)->GetStringUTFChars(env, string, 0);
    jint rcode;
    currentEnv = env;
    rcode = (jint) xsb_command_string(command);
    (*env)->ReleaseStringUTFChars(env, string, command);
    return rcode;
}

/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_query_string
 * Signature: (Ljava/lang/String;)I
 */

JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1query_1string
    (JNIEnv * env, jobject obj, jstring string){
    char * query = (char *) (*env)->GetStringUTFChars(env, string, 0);
    jint rcode;
    currentEnv = env;
    rcode = (jint) xsb_query_string(query);
    (*env)->ReleaseStringUTFChars(env, string, query);
    return rcode;
}

/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_close_query
 * Signature: ()I
 */

JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1close_1query
    (JNIEnv * env, jobject obj){

    jint rcode = (jint) xsb_close_query();
}


/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_answer_string
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */

JNIEXPORT jstring JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1answer_1string
    (JNIEnv * env, jobject obj, jstring string){
    XSB_StrDefine(ans);
    char * sep = (char *) (*env)->GetStringUTFChars(env, string, 0);
    jstring answer;
    xsb_answer_string(& ans, sep);
    answer = (*env)->NewStringUTF(env, ans.string);
    XSB_StrDestroy(&ans); 
    (*env)->ReleaseStringUTFChars(env, string, sep);
    return answer;
}


/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_next
 * Signature: ()I
 */


JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1next
    (JNIEnv * env, jobject obj){
    jint rcode;
    currentEnv = env;
    rcode = (jint) xsb_next();
    return rcode;
}


/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_command
 * Signature: ()I
 */

JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1command
(JNIEnv * env, jobject obj){
    jint rcode;
    currentEnv = env;
    rcode = (jint) xsb_command();
    return rcode;
}


/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_query
 * Signature: ()I
 */

JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1query
(JNIEnv * env, jobject obj){
    jint rcode;
    currentEnv = env;
    rcode = (jint) xsb_query();
    return rcode;
}


/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_getAnswerSubstitution
 * Signature: ()[Ledu/stanford/db/lp/Term;
 */

JNIEXPORT jobjectArray JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1getAnswerSubstitution
(JNIEnv * env, jobject obj){
	int i;
	jsize len = (jsize) p2c_arity(reg_term(2));
	jobjectArray ret;
	ret= (jobjectArray) (*env)->NewObjectArray( env,len,
		(*env)->FindClass(env,"edu/stanford/db/lp/Term"),
                NULL);
	for (i=1; i<=len; i++) {
		jobject jterm = prolog2java(p2p_arg(reg_term(2),i),env);
		(*env)->SetObjectArrayElement(env, ret, i-1, jterm); 
  	}
	return ret;
}



void getLiteral(JNIEnv * env , jobject literal, HashTable * hashTable, prolog_term var){
  char * functor;
  jint rcode;
  jstring jfunctor;
  jobject args;
  jsize len;
  int k;
  prolog_term term;
  jboolean isAtomic;
  jfunctor = (jstring) (*env)->CallObjectMethod(env, literal, getPredicateSymbolLiteral);
  isAtomic =  (*env)->CallBooleanMethod(env, literal, isAtomLiteral);
  functor = (char *) (*env)->GetStringUTFChars(env, jfunctor , 0);
  args =  (*env)->CallObjectMethod(env, literal, getArgsLiteral);
  len = (*env)->GetArrayLength(env, args); 
  if(!isAtomic) {
     c2p_functor("sk_not",1,var);
     term = p2p_arg(var,1);
  }
  else term = var;
  c2p_functor(functor,len,term);  
  for(k=0;k<len;k++){
    jobject argument = (*env)->GetObjectArrayElement(env, args, (jsize) k );
    java2prolog(env, argument,p2p_arg(term,k+1),hashTable);
  }
  (*env)->ReleaseStringUTFChars(env, jfunctor , functor);
}



jint commandRule( char * command, JNIEnv * env , jobject clause){
  jint rcode = 0;
  jobject head;
  jobject body;
  jobject literal, literal2;
  int bodylength;
  int headlength; 
  HashTable * hashTable;
  prolog_term term, actbody, bodyt;
  int i,k;
  head = (*env)-> CallObjectMethod(env, clause, getHeadProgramClause);
  body =  (*env)-> CallObjectMethod(env, clause, getBodyProgramClause);
  headlength = (*env)->GetArrayLength(env, head);
  bodylength = (*env)->GetArrayLength(env, body);

  for(i=0;i<headlength;i++){
      hashTable = initVarHashTable();
      c2p_functor(command,1,reg_term(1));
      literal = (*env)->GetObjectArrayElement(env, head, (jsize) i );
      term = p2p_arg(reg_term(1),1);
      c2p_functor(":-",2,term);
      getLiteral(env, literal, hashTable, p2p_arg(term,1));
      bodyt= p2p_arg(term,2);

      actbody = bodyt;
      for(k=0;k<bodylength;k++){
	literal2 = (*env)->GetObjectArrayElement(env, body, (jsize) k );
	if(k< bodylength-1) { 
	  c2p_functor(",",2,actbody);
	  getLiteral(env, literal2, hashTable, p2p_arg(actbody,1));
	  actbody = p2p_arg(actbody,2);
	}
	else
	  getLiteral(env, literal2 , hashTable, actbody);
      }
     rcode = rcode | xsb_command();
     HashTableDestroy(hashTable);
  }
  return rcode;
}
  


/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_assert_rule
 * Signature: (Ledu/stanford/db/lp/ProgramClause;)I
 */
JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1assert_1rule
(JNIEnv * env, jobject obj, jobject clause){
  commandRule( "assert", env , clause);
}

/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_retract_rule
 * Signature: (Ledu/stanford/db/lp/ProgramClause;)I
 */
JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1retract_1rule
(JNIEnv * env, jobject obj, jobject clause){
    commandRule( "retract", env , clause);
}             




/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_assert_fact
 * Signature: (Ledu/stanford/db/lp/Literal;)I
 */




jint commandFact( char * command, JNIEnv * env , jobject fact){
  char * functor;
  jstring jfunctor;
  jobject args;
  jsize len;
  int k;
  prolog_term term;
  HashTable * hashTable;
  jfunctor = (jstring) (*env)->CallObjectMethod(env, fact, getPredicateSymbolLiteral);
  functor = (char *) (*env)->GetStringUTFChars(env, jfunctor , 0);
  args =  (*env)->CallObjectMethod(env, fact, getArgsLiteral);
  len = (*env)->GetArrayLength(env, args); 
 
  c2p_functor(command,1,reg_term(1));
 
  term = p2p_arg(reg_term(1),1);
  c2p_functor(functor,len,term);
  hashTable = initVarHashTable();
  
  for(k=0;k<len;k++){
    jobject argument = (*env)->GetObjectArrayElement(env, args, (jsize) k );
    java2prolog(env, argument,p2p_arg(term,k+1),hashTable);
  }
  (*env)->ReleaseStringUTFChars(env, jfunctor , functor);
  HashTableDestroy(hashTable);
  return (jint) xsb_command();
}

JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1assert_1fact
(JNIEnv * env, jobject obj, jobject fact){
    return commandFact("assert", env , fact);
}

/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_retract_fact
 * Signature: (Ledu/stanford/db/lp/Literal;)I
 */
JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1retract_1fact
(JNIEnv * env , jobject obj, jobject fact){
    return commandFact("retract", env , fact);
}

/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_query_java
 * Signature: (Ledu/stanford/db/lp/ProgramClause;)I
 */
JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1query_1java
(JNIEnv * env, jobject obj, jobject clause ){

  jint rcode;
  jobject body;
  jobject literal;
  int bodylength;
  HashTable * hashTable;
  prolog_term actbody;
  int k;
  currentEnv = env;
  body =  (*env)-> CallObjectMethod(env, clause, getBodyProgramClause);
  bodylength = (*env)->GetArrayLength(env, body);
  hashTable = initVarHashTable();
  actbody = reg_term(1);
  for(k=0;k<bodylength;k++){
    literal = (*env)->GetObjectArrayElement(env, body, (jsize) k );
    if(k< bodylength-1) { 
      c2p_functor(",",2,actbody);
      getLiteral(env, literal, hashTable, p2p_arg(actbody,1));
      actbody = p2p_arg(actbody,2);
    }
    else
      getLiteral(env, literal, hashTable, actbody);
  }
  rcode = xsb_query();
  HashTableDestroy(hashTable);
  return rcode;
}

/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_query_term
 * Signature: (Ledu/stanford/db/lp/Term;)I
 */
JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1query_1term
(JNIEnv * env, jobject obj, jobject jterm){
  HashTable * hashTable = initVarHashTable();
  currentEnv = env;
  java2prolog(env, jterm,reg_term(1),hashTable);
  HashTableDestroy(hashTable);
  return (jint) xsb_query();
}


/*
 * Class:     edu_stanford_db_xsb_XSBCore
 * Method:    xsb_command_term
 * Signature: (Ledu/stanford/db/lp/Term;)I
 */
JNIEXPORT jint JNICALL Java_edu_stanford_db_xsb_XSBCore_xsb_1command_1term
(JNIEnv * env, jobject obj, jobject jterm){
  HashTable * hashTable = initVarHashTable();
  currentEnv = env;
  java2prolog(env,jterm,reg_term(1),hashTable);
  HashTableDestroy(hashTable);
  return (jint) xsb_command();
}













