#include <xsb_config.h>
#include <cell_xsb.h> 

#include <jni.h>
#include "edu_stanford_db_xsb_XSBCore.h"
#include <stdio.h>
#include <cinterf.h>
#include "javaxsbdefs.h"
#include <basicdefs.h>

char tempstring[MAXBUFSIZE];

jobject prolog2java(prolog_term term, JNIEnv * env){
  int i;
 
  if (is_var(term)) { 
    jstring stringarg;
    /* get string representation of variable */
    xsb_sprint_variable(tempstring, (CPtr) term);

    /* turn string into a jstring */
    stringarg = (*env)->NewStringUTF(env, tempstring);
   
    /* create and return obj with constructor */
     return (*env)->NewObject(env, clsVariable, clsVariableConstructor, stringarg); 

  } else if (is_int(term)) {
     /* get integer value of term */
       jint intarg = (jint) p2c_int(term);

    /* create and return obj with constructor */
    return (*env)->NewObject(env, clsInteger, clsIntegerConstructor, intarg); 

  } else if (is_float(term)) {
     /* get float value of term */
    jdouble doublearg = (jdouble) p2c_float(term);

    /* create and return obj with constructor */
    return (*env)->NewObject(env, clsDouble, clsDoubleConstructor, doublearg); 

  } else if (is_nil(term)) {

    /* create and return  obj with constructor */
    return (*env)->NewObject(env, clsNil, clsNilConstructor); 

  } else if (is_string(term)) {
    jstring stringarg;
    /* turn string into a jstring */
    stringarg = (*env)->NewStringUTF(env, p2c_string(term));

    /* Find Class representing a StringTerm */
    /*cls = (*env)->FindClass(env,"edu/stanford/db/lp/StringTerm");*/
    /* My missunderstanding: a StringTerm in XSB is a constant! */

    /* create and return obj with constructor */
    return (*env)->NewObject(env, clsConst, clsConstConstructor, stringarg); 

  } else if(is_list(term)) { 
    jobject result;  
    jobject list;
    jobject newlist;
    jobject nobj;

    /* create new result list */
    result = (*env)->NewObject(env, clsList, clsConstConstructor);

    /* which is initially the same as the list used in to loop */
    list = result;

    /* get Java representation for first element of list */
    nobj = prolog2java(p2p_car(term), env);

    /* set the head of the result list */
    (*env)->CallObjectMethod(env,list,setHeadList,nobj); 

     term = p2p_cdr(term);
       while (is_list(term)) {

       /* create a new list - will the be tail of the former list */
       newlist = (*env)->NewObject(env, clsList, clsListConstructor);

       /* set the rest */
       (*env)->CallObjectMethod(env,list,setRestList,newlist);

       /* switch lists */
       list = newlist;

       /* compute the java representation of the actual head of the list */
        nobj = prolog2java(p2p_car(term), env);

       /* set the head of the list */
       (*env)->CallObjectMethod(env,list,setHeadList,nobj); 

       /* and go the the next iteration */
       term = p2p_cdr(term);
    }

    /* the following code is independet of the fact, is term is a 
       NIL-Term or not. If it is, prolog2java will return a nil-term,
       if not it will return another term.
    */
    nobj = prolog2java(term, env);
    (*env)->CallObjectMethod(env,list,setRestList,nobj);
    return result;

  } else if (is_functor(term)) { 
      int i;
      jstring functor = (*env)->NewStringUTF(env, p2c_functor(term));
      jsize arity = (jsize) p2c_arity(term);
      if(arity==0) {
         /* return const */
        /* create and return with constructor */
            return (*env)->NewObject(env, clsConst, clsConstConstructor, functor); 

      } else

      {
	
	jobjectArray args;
	args= (jobjectArray) (*env)->NewObjectArray(env, arity,
		(*env)->FindClass(env,"edu/stanford/db/lp/Term"),
                NULL);
	for (i=1; i<=arity; i++) {
	
		jobject jterm = prolog2java(p2p_arg(term,i),env);
		
		(*env)->SetObjectArrayElement(env, args, i-1, jterm); 
  	}
       /* create obj with constructor */
       return (*env)->NewObject(env, clsFunctor, clsFunctorConstructor, functor, args); 
      }
  }  else  { xsb_warn("prolog2java: Unrecognized prolog term type");
    return (*env)->NewObject(env, clsInteger, clsIntegerConstructor, 9999);}
}













